from typing import List

import attr

from qaseio.client.models.base import DefaultFilter, DefaultList


@attr.s
class MilestoneCreate:
    title: str = attr.ib()
    description: str = attr.ib(default=None)


@attr.s
class MilestoneUpdate:
    title: str = attr.ib(default=None)
    description: str = attr.ib(default=None)


@attr.s
class MilestoneCreated:
    id = attr.ib(default=None)


@attr.s
class MilestoneInfo:
    id = attr.ib(default=None)
    title = attr.ib(default=None)
    description = attr.ib(default=None)
    due_date = attr.ib(default=None)
    created = attr.ib(default=None)
    updated = attr.ib(default=None)


@attr.s
class MilestoneList(DefaultList):
    entities: List[MilestoneInfo] = attr.ib(factory=list)


@attr.s
class MilestoneFilters(DefaultFilter):
    search: str = attr.ib(default=None)



from typing import Union

from qaseio.client.models import UserInfo, UserList
from qaseio.client.services import BaseService, NotFoundException


class Users(BaseService):
    def get_all(self, limit=None, offset=None):
        return self.vr(
            self.s.get(
                self.path("user"), params={"limit": limit, "offset": offset}
            ),
            to_type=UserList,
        )

    def get(self, user_id: Union[str, int]):
        return self.vr(
            self.s.get(self.path("user/{}".format(user_id))),
            to_type=UserInfo,
        )

    def exists(self, code: str):
        try:
            return self.get(code)
        except NotFoundException:
            return False



import pytest

from qaseio.client import QaseApi
from qaseio.client.services import BaseService


@pytest.fixture()
def client():
    api = QaseApi("")
    api.projects._in_test = True
    api.cases._in_test = True
    api.runs._in_test = True
    api.results._in_test = True
    api.plans._in_test = True
    api.suites._in_test = True
    api.milestones._in_test = True
    api.shared_steps._in_test = True
    api.defects._in_test = True
    api.attachments._in_test = True
    api.custom_fields._in_test = True
    api.users._in_test = True
    return api


@pytest.fixture()
def base_service():
    return BaseService(None, None)



from typing import Union

from qaseio.client.models import (
    TestPlanCreate,
    TestPlanCreated,
    TestPlanInfo,
    TestPlanList,
    TestPlanUpdate,
)
from qaseio.client.services import BaseService, NotFoundException


class Plans(BaseService):
    def get_all(
        self,
        code: str,
        limit=None,
        offset=None,
    ):
        query = {"limit": limit, "offset": offset}
        return self.vr(
            self.s.get(self.path("plan/{}".format(code)), params=query),
            to_type=TestPlanList,
        )

    def get(self, code: str, plan_id: Union[str, int]):
        return self.vr(
            self.s.get(self.path("plan/{}/{}".format(code, plan_id))),
            to_type=TestPlanInfo,
        )

    def create(self, code: str, data: TestPlanCreate):
        return self.vr(
            self.s.post(self.path("plan/{}".format(code)), data=data),
            to_type=TestPlanCreated,
        )

    def update(
        self, code: str, plan_id: Union[str, int], data: TestPlanUpdate
    ):
        return self.vr(
            self.s.patch(
                self.path("plan/{}/{}".format(code, plan_id)), data=data
            ),
            to_type=TestPlanCreated,
        )

    def delete(self, code: str, plan_id: Union[str, int]):
        return self.vr(
            self.s.delete(self.path("plan/{}/{}".format(code, plan_id))),
            to_type=None,
        )

    def exists(self, code: str, plan_id: Union[str, int]):
        try:
            return self.get(code, plan_id)
        except NotFoundException:
            return False



from enum import Enum, unique
from typing import List

import attr

from qaseio.client.models.base import DefaultFilter, DefaultList


@unique
class TestRunStatus(Enum):
    ACTIVE = "active"
    COMPLETE = "complete"
    ABORT = "abort"


class TestRunInclude:
    NONE = None
    CASES = "cases"


@attr.s
class TestRunCreate:
    title: str = attr.ib()
    cases: List[int] = attr.ib()
    description: str = attr.ib(default=None)
    environment_id: int = attr.ib(default=None)

    @cases.validator
    def check(self, _, value):
        if value is not None and not isinstance(value, list):
            raise ValueError("You should provide list of cases or None")


@attr.s
class TestRunCreated:
    id = attr.ib(default=None)


@attr.s
class TestRunInfoStats:
    total = attr.ib(default=None)
    untested = attr.ib(default=None)
    passed = attr.ib(default=None)
    failed = attr.ib(default=None)
    blocked = attr.ib(default=None)
    skipped = attr.ib(default=None)
    retest = attr.ib(default=None)
    deleted = attr.ib(default=None)


@attr.s
class TestRunInfo:
    id = attr.ib(default=None)
    title = attr.ib(default=None)
    description = attr.ib(default=None)
    status = attr.ib(default=None)
    start_time = attr.ib(default=None)
    end_time = attr.ib(default=None)
    public = attr.ib(default=None)
    stats: TestRunInfoStats = attr.ib(default=None)
    time_spent = attr.ib(default=None)
    user_id = attr.ib(default=None)
    environment = attr.ib(default=None)
    cases = attr.ib(factory=list)


@attr.s
class TestRunFilters(DefaultFilter):
    status: List[TestRunStatus] = attr.ib(default=None)


@attr.s
class TestRunList(DefaultList):
    entities: List[TestRunInfo] = attr.ib(factory=list)



from typing import List

import attr

from qaseio.client.models.base import AccessLevel, DefaultList


@attr.s
class ProjectCreate:
    title: str = attr.ib()
    code: str = attr.ib(
        validator=[
            attr.validators.instance_of(str),
            attr.validators.matches_re(r"[a-zA-Z]{2,6}"),
        ]
    )
    description: str = attr.ib(default=None)
    access: AccessLevel = attr.ib(default=AccessLevel.NONE)
    group: str = attr.ib(default=None)

    @group.validator
    def check(self, _, value):
        if self.access == AccessLevel.GROUP and not value:
            raise ValueError(
                "Group hash should be provided access group level"
            )


@attr.s
class ProjectCreated:
    code = attr.ib(default=None)


@attr.s
class ProjectCountsRuns:
    total = attr.ib(default=None)
    active = attr.ib(default=None)


@attr.s
class ProjectCountsDefects:
    total = attr.ib(default=None)
    open = attr.ib(default=None)


@attr.s
class ProjectCounts:
    cases = attr.ib(default=None)
    suites = attr.ib(default=None)
    milestones = attr.ib(default=None)
    runs: ProjectCountsRuns = attr.ib(default=None)
    defects: ProjectCountsDefects = attr.ib(default=None)


@attr.s
class ProjectInfo:
    title = attr.ib(default=None)
    code = attr.ib(default=None)
    counts: ProjectCounts = attr.ib(default=None)


@attr.s
class ProjectList(DefaultList):
    entities: List[ProjectInfo] = attr.ib(factory=list)



from qaseio.client.models import (
    SharedStepCreate,
    SharedStepCreated,
    SharedStepFilters,
    SharedStepInfo,
    SharedStepList,
    SharedStepUpdate,
)
from qaseio.client.services import BaseService, NotFoundException


class SharedSteps(BaseService):
    def get_all(
        self,
        code: str,
        limit=None,
        offset=None,
        filters: SharedStepFilters = None,
    ):
        query = {"limit": limit, "offset": offset}
        if filters:
            query.update(filters.filter())
        return self.vr(
            self.s.get(self.path("shared_step/{}".format(code)), params=query),
            to_type=SharedStepList,
        )

    def get(self, code: str, shared_step_hash: str):
        return self.vr(
            self.s.get(
                self.path("shared_step/{}/{}".format(code, shared_step_hash))
            ),
            to_type=SharedStepInfo,
        )

    def create(self, code: str, data: SharedStepCreate):
        return self.vr(
            self.s.post(self.path("shared_step/{}".format(code)), data=data),
            to_type=SharedStepCreated,
        )

    def update(
        self,
        code: str,
        shared_step_hash: str,
        data: SharedStepUpdate,
    ):
        return self.vr(
            self.s.patch(
                self.path("shared_step/{}/{}".format(code, shared_step_hash)),
                data=data,
            ),
            to_type=SharedStepCreated,
        )

    def delete(self, code: str, shared_step_hash: str):
        return self.vr(
            self.s.delete(
                self.path("shared_step/{}/{}".format(code, shared_step_hash))
            ),
            to_type=None,
        )

    def exists(self, code: str, shared_step_hash: str):
        try:
            return self.get(code, shared_step_hash)
        except NotFoundException:
            return False



import json

import pytest

import requests_mock
from apitist.constructor import converter
from tests.data import _list, _project, _status_true

from qaseio.client.models import (
    ProjectCreate,
    ProjectCreated,
    ProjectInfo,
    ProjectList,
)


@pytest.mark.parametrize(
    "params, query",
    [
        ((10, 30), "?limit=10&offset=30"),
        ((None, 30), "?offset=30"),
        ((10, None), "?limit=10"),
    ],
)
def test_get_all_projects(client, params, query):
    response = _status_true(_list(_project()))
    with requests_mock.Mocker() as m:
        m.get(client._path("project"), json=response)
        data = client.projects.get_all(*params)
        assert data == converter.structure(response.get("result"), ProjectList)
        res = client.projects._last_res
        assert res.url == client._path("project" + query)


def test_get_specific_project(client):
    response = _status_true(_project())
    with requests_mock.Mocker() as m:
        m.get(client._path("project/CODE"), json=response)
        data = client.projects.get("CODE")
        assert data == converter.structure(response.get("result"), ProjectInfo)
        res = client.projects._last_res
        assert res.url == client._path("project/CODE")


def test_project_exists(client):
    response = _status_true(_project())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("project/CODE"),
            [
                {"status_code": 200, "json": response},
                {"status_code": 404, "json": {}},
            ],
        )
        assert client.projects.exists("CODE")
        assert not client.projects.exists("CODE")


def test_create_new_project(client):
    response = _status_true({"code": "NEW"})
    with requests_mock.Mocker() as m:
        m.post(client._path("project"), json=response)
        create_data = ProjectCreate("new project", "NEW")
        data = client.projects.create(create_data)
        assert data == converter.structure(
            response.get("result"), ProjectCreated
        )
        res = client.projects._last_res
        assert json.loads(res.request.body) == converter.unstructure(
            create_data
        )



from typing import List

import attr

from qaseio.client.models.base import DefaultFilter, DefaultList


@attr.s
class TestSuiteCreate:
    title: str = attr.ib()
    description: str = attr.ib(default=None)
    preconditions: str = attr.ib(default=None)
    parent_id: int = attr.ib(default=None)


@attr.s
class TestSuiteUpdate(TestSuiteCreate):
    title: str = attr.ib(default=None)


@attr.s
class TestSuiteCreated:
    id = attr.ib(default=None)


@attr.s
class TestSuiteInfo:
    id = attr.ib(default=None)
    title = attr.ib(default=None)
    description = attr.ib(default=None)
    preconditions = attr.ib(default=None)
    position: int = attr.ib(default=None)
    cases_count: int = attr.ib(default=None)
    parent_id: int = attr.ib(default=None)
    created = attr.ib(default=None)
    updated = attr.ib(default=None)


@attr.s
class TestSuiteList(DefaultList):
    entities: List[TestSuiteInfo] = attr.ib(factory=list)


@attr.s
class TestSuiteFilters(DefaultFilter):
    search: str = attr.ib(default=None)



from typing import Union

from qaseio.client.models import (
    TestRunCreate,
    TestRunCreated,
    TestRunFilters,
    TestRunInclude,
    TestRunInfo,
    TestRunList,
)
from qaseio.client.services import BaseService, NotFoundException


class Runs(BaseService):
    def get_all(
        self,
        code: str,
        limit=None,
        offset=None,
        include=TestRunInclude.NONE,
        filters: TestRunFilters = None,
    ):
        query = {"limit": limit, "offset": offset, "include": include}
        if filters:
            query.update(filters.filter())
        return self.vr(
            self.s.get(self.path("run/{}".format(code)), params=query),
            to_type=TestRunList,
        )

    def get(
        self, code: str, run_id: Union[str, int], include=TestRunInclude.NONE
    ):
        query = {"include": include}
        return self.vr(
            self.s.get(
                self.path("run/{}/{}".format(code, run_id)), params=query
            ),
            to_type=TestRunInfo,
        )

    def create(self, code: str, data: TestRunCreate):
        return self.vr(
            self.s.post(self.path("run/{}".format(code)), data=data),
            to_type=TestRunCreated,
        )

    def delete(self, code: str, run_id: Union[str, int]):
        return self.vr(
            self.s.delete(self.path("run/{}/{}".format(code, run_id))),
            to_type=None,
        )

    def exists(self, code: str, run_id: Union[str, int], **kwargs):
        try:
            return self.get(code, run_id, **kwargs)
        except NotFoundException:
            return False



from typing import Union

from qaseio.client.models import CustomFieldInfo, CustomFieldList
from qaseio.client.services import BaseService, NotFoundException


class CustomFields(BaseService):
    def get_all(self, code: str, limit=None, offset=None):
        return self.vr(
            self.s.get(
                self.path("custom_field/{}".format(code)),
                params={"limit": limit, "offset": offset},
            ),
            to_type=CustomFieldList,
        )

    def get(self, code: str, field_id: Union[str, int]):
        return self.vr(
            self.s.get(self.path("custom_field/{}/{}".format(code, field_id))),
            to_type=CustomFieldInfo,
        )

    def exists(self, code: str, field_id: Union[str, int]):
        try:
            return self.get(code, field_id)
        except NotFoundException:
            return False



from typing import List

import attr

from qaseio.client.models.base import DefaultList


@attr.s
class CustomFieldInfo:
    id = attr.ib(default=None)
    title = attr.ib(default=None)
    type = attr.ib(default=None)
    placeholder = attr.ib(default=None)
    default_value = attr.ib(default=None)
    value = attr.ib(default=None)
    is_required: bool = attr.ib(default=None)
    is_visible: bool = attr.ib(default=None)
    is_filterable: bool = attr.ib(default=None)
    created = attr.ib(default=None)
    updated = attr.ib(default=None)


@attr.s
class CustomFieldList(DefaultList):
    entities: List[CustomFieldInfo] = attr.ib(factory=list)



import json

import pytest

import requests_mock
from apitist.constructor import converter
from tests.data import _list, _status_true, _test_suite

from qaseio.client.models import (
    MilestoneCreate,
    MilestoneCreated,
    MilestoneFilters,
    MilestoneInfo,
    MilestoneList,
    MilestoneUpdate,
)


@pytest.mark.parametrize(
    "params, query",
    [
        ((10, 30), "?limit=10&offset=30"),
        ((None, 30), "?offset=30"),
        ((10, None), "?limit=10"),
        (
            (
                10,
                None,
                MilestoneFilters(search="123"),
            ),
            "?limit=10&filters%5Bsearch%5D=123",
        ),
    ],
)
def test_get_all_test_suites(client, params, query):
    response = _status_true(_list(_test_suite()))
    with requests_mock.Mocker() as m:
        m.get(client._path("milestone/CODE"), json=response)
        data = client.milestones.get_all("CODE", *params)
        assert data == converter.structure(
            response.get("result"), MilestoneList
        )
        res = client.milestones._last_res
        assert res.url == client._path("milestone/CODE" + query)


def test_get_specific_test_suite(client):
    response = _status_true(_test_suite())
    with requests_mock.Mocker() as m:
        m.get(client._path("milestone/CODE/123"), json=response)
        data = client.milestones.get("CODE", 123)
        assert data == converter.structure(
            response.get("result"), MilestoneInfo
        )
        res = client.milestones._last_res
        assert res.url == client._path("milestone/CODE/123")


def test_test_suite_exists(client):
    response = _status_true(_test_suite())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("milestone/CODE/123"),
            [
                {"status_code": 200, "json": response},
                {"status_code": 404, "json": {}},
            ],
        )
        assert client.milestones.exists("CODE", 123)
        assert not client.milestones.exists("CODE", 123)


def test_create_new_test_suite(client):
    response = _status_true({"id": 123})
    with requests_mock.Mocker() as m:
        m.post(client._path("milestone/CODE"), json=response)
        create_data = MilestoneCreate("new test suite")
        data = client.milestones.create("CODE", create_data)
        assert data == converter.structure(
            response.get("result"), MilestoneCreated
        )
        res = client.milestones._last_res
        assert json.loads(res.request.body) == converter.unstructure(
            create_data
        )


def test_update_test_suite(client):
    response = _status_true({"id": 123})
    with requests_mock.Mocker() as m:
        m.patch(client._path("milestone/CODE/123"), json=response)
        update_data = MilestoneUpdate("new test plan")
        data = client.milestones.update("CODE", 123, update_data)
        assert data == converter.structure(
            response.get("result"), MilestoneCreated
        )
        res = client.milestones._last_res
        assert res.url == client._path("milestone/CODE/123")
        assert json.loads(res.request.body) == converter.unstructure(
            update_data
        )


def test_delete_test_suite(client):
    with requests_mock.Mocker() as m:
        m.delete(client._path("milestone/CODE/123"), json={"status": True})
        data = client.milestones.delete("CODE", 123)
        assert data is None
        res = client.milestones._last_res
        assert res.url == client._path("milestone/CODE/123")



# -*- coding: utf-8 -*-
import logging
import time
from functools import partial
from pkg_resources import DistributionNotFound, get_distribution
from typing import Callable
from urllib.parse import urljoin

import attr
from apitist.hooks import (
    PrepRequestDebugLoggingHook,
    RequestConverterHook,
    ResponseDebugLoggingHook,
    ResponseHook,
)
from apitist.requests import Session
from requests import Response

from qaseio.client.services.attachments import Attachments
from qaseio.client.services.cases import Cases
from qaseio.client.services.custom_fields import CustomFields
from qaseio.client.services.defects import Defects
from qaseio.client.services.milestones import Milestones
from qaseio.client.services.plans import Plans
from qaseio.client.services.projects import Projects
from qaseio.client.services.results import Results
from qaseio.client.services.runs import Runs
from qaseio.client.services.shared_steps import SharedSteps
from qaseio.client.services.suites import Suites
from qaseio.client.services.users import Users

try:
    # Change here if project is renamed and does not equal the package name
    dist_name = "qaseio"
    __version__ = get_distribution(dist_name).version
except DistributionNotFound:
    __version__ = "unknown"
finally:
    del get_distribution, DistributionNotFound


logger = logging.getLogger("qase-api")


@attr.s
class QaseApi:
    api_token: str = attr.ib(repr=False)
    _s: Session = attr.ib(factory=lambda: Session(), repr=False, init=False)
    _path: Callable[[str], str] = attr.ib(repr=False, init=False)
    projects: Projects = attr.ib(init=False)
    cases: Cases = attr.ib(init=False)
    runs: Runs = attr.ib(init=False)
    results: Results = attr.ib(init=False)
    plans: Plans = attr.ib(init=False)
    suites: Suites = attr.ib(init=False)
    milestones: Milestones = attr.ib(init=False)
    shared_steps: SharedSteps = attr.ib(init=False)
    defects: Defects = attr.ib(init=False)
    attachments: Attachments = attr.ib(init=False)
    custom_fields: CustomFields = attr.ib(init=False)
    users: Users = attr.ib(init=False)

    def __attrs_post_init__(self):
        class ResponseRetryAfterLimitHook(ResponseHook):
            def run(_self, response: Response) -> Response:
                nonlocal self
                if response.status_code == 429:
                    retry_after = int(response.headers.get("retry-after", 60))
                    logger.warning(
                        "qase: got 429 for {}, sleeping for {}".format(
                            response.url, retry_after
                        )
                    )
                    time.sleep(retry_after)
                    response = self._s.send(response.request)
                return response

        self._s.add_request_hook(RequestConverterHook)
        self._s.add_prep_request_hook(PrepRequestDebugLoggingHook)
        self._s.add_response_hook(ResponseDebugLoggingHook)
        self._s.add_response_hook(ResponseRetryAfterLimitHook)
        self._s.headers.update({"Token": self.api_token})

        def get_url(path: str):
            if isinstance(path, str) and path.startswith("/"):
                path = path[1:]
            return partial(urljoin, "https://api.qase.io/v1/")(path)

        self._path = get_url

        self.projects = Projects(self._s, self._path)
        self.cases = Cases(self._s, self._path)
        self.runs = Runs(self._s, self._path)
        self.results = Results(self._s, self._path)
        self.plans = Plans(self._s, self._path)
        self.suites = Suites(self._s, self._path)
        self.milestones = Milestones(self._s, self._path)
        self.shared_steps = SharedSteps(self._s, self._path)
        self.defects = Defects(self._s, self._path)
        self.attachments = Attachments(self._s, self._path)
        self.custom_fields = CustomFields(self._s, self._path)
        self.users = Users(self._s, self._path)



from enum import Enum, unique

import attr


@unique
class AccessLevel(Enum):
    ALL = "all"
    GROUP = "group"
    NONE = "none"


@unique
class Severity(Enum):
    UNDEFINED = "undefined"
    BLOCKER = "blocker"
    CRITICAL = "critical"
    MAJOR = "major"
    NORMAL = "normal"
    MINOR = "minor"
    TRIVIAL = "trivial"


@unique
class Priority(Enum):
    UNDEFINED = "undefined"
    HIGH = "high"
    MEDIUM = "medium"
    LOW = "low"


@unique
class Type(Enum):
    OTHER = "other"
    FUNCTIONAL = "functional"
    SMOKE = "smoke"
    REGRESSION = "regression"
    SECURITY = "security"
    USABILITY = "usability"
    PERFORMANCE = "performance"
    ACCEPTANCE = "acceptance"


@unique
class Behavior(Enum):
    UNDEFINED = "undefined"
    POSITIVE = "positive"
    NEGATIVE = "negative"
    DESTRUCTIVE = "destructive"


@unique
class Automation(Enum):
    IS_NOT_AUTOMATED = "is-not-automated"
    AUTOMATED = "automated"
    TO_BE_AUTOMATED = "to-be-automated"


@attr.s
class DefaultList:
    total: int = attr.ib(default=None)
    filtered: int = attr.ib(default=None)
    count: int = attr.ib(default=None)


@attr.s
class DefaultFilter:
    _is_filter = True

    def _no_enum(self, value):
        if isinstance(value, Enum):
            return value.value
        return value

    def filter(self):
        from apitist.utils import is_sequence

        fields = attr.fields(type(self))
        filters = {}
        for field in fields:
            name = field.name
            values = getattr(self, field.name)
            if (
                values is not None
                and is_sequence(field.type)
                and isinstance(values, list)
            ):
                values = ",".join([self._no_enum(val) for val in values])
            else:
                values = self._no_enum(values)
            if values:
                filters[f"filters[{name}]"] = values
        return filters



import pytest

import requests_mock
from apitist.constructor import converter
from requests_toolbelt.multipart import decoder
from tests.data import _attachment, _attachment_created, _list, _status_true

from qaseio.client.models import (
    AttachmentCreated,
    AttachmentInfo,
    AttachmentList,
    List,
    MimeTypes,
)


@pytest.mark.parametrize(
    "params, query",
    [
        ((10, 30), "?limit=10&offset=30"),
        ((None, 30), "?offset=30"),
        ((10, None), "?limit=10"),
    ],
)
def test_get_all_attachments(client, params, query):
    response = _status_true(_list(_attachment()))
    with requests_mock.Mocker() as m:
        m.get(client._path("attachment"), json=response)
        data = client.attachments.get_all(*params)
        assert data == converter.structure(
            response.get("result"), AttachmentList
        )
        res = client.attachments._last_res
        assert res.url == client._path("attachment" + query)


def test_get_specific_attachment(client):
    response = _status_true(_attachment())
    with requests_mock.Mocker() as m:
        m.get(client._path("attachment/1q2w3e"), json=response)
        data = client.attachments.get("1q2w3e")
        assert data == converter.structure(
            response.get("result"), AttachmentInfo
        )
        res = client.attachments._last_res
        assert res.url == client._path("attachment/1q2w3e")


def test_attachment_exists(client):
    response = _status_true(_attachment())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("attachment/1q2w3e"),
            [
                {"status_code": 200, "json": response},
                {"status_code": 404, "json": {}},
            ],
        )
        assert client.attachments.exists("1q2w3e")
        assert not client.attachments.exists("1q2w3e")


@pytest.mark.parametrize(
    "files",
    [
        (("./tests/test_data/example.json", MimeTypes.JSON, "example.json"),),
        (("./tests/test_data/example.somemime", None, "example.somemime"),),
        (
            (
                ("./tests/test_data/example.json", "text/plain"),
                MimeTypes.TXT,
                "example.json",
            ),
        ),
        (("./tests/test_data/example.png", MimeTypes.PNG, "example.png"),),
        (
            (
                (b"some content", MimeTypes.CSV, "some.csv"),
                MimeTypes.CSV,
                "some.csv",
            ),
        ),
        (
            ("./tests/test_data/example.json", MimeTypes.JSON, "example.json"),
            ("./tests/test_data/example.somemime", None, "example.somemime"),
            (
                ("./tests/test_data/example.json", "text/plain"),
                MimeTypes.TXT,
                "example.json",
            ),
            ("./tests/test_data/example.png", MimeTypes.PNG, "example.png"),
            (
                (b"some content", MimeTypes.CSV, "some.csv"),
                MimeTypes.CSV,
                "some.csv",
            ),
        ),
    ],
)
def test_upload_attachements(client, files):
    response = _status_true([_attachment_created()])
    with requests_mock.Mocker() as m:
        m.post(client._path("attachment/CODE"), json=response)
        data = client.attachments.upload(
            "CODE", *[file for file, _, _ in files]
        )
        assert data == converter.structure(
            response.get("result"), List[AttachmentCreated]
        )
        res = client.attachments._last_res
        assert res.request.body
        data = decoder.MultipartDecoder(
            res.request.body, res.request.headers["content-type"]
        )
        for i, part in enumerate(data.parts):
            file, mime, filename = files[i]
            disposition = 'form-data; name="{}"; filename="{}"'.format(
                i, filename
            )
            assert part.headers[b"content-disposition"] == disposition.encode(
                "utf-8"
            )
            if mime:
                assert part.headers[b"content-type"] == mime.encode("utf-8")
            assert part.content


def test_delete_attachment(client):
    with requests_mock.Mocker() as m:
        m.delete(client._path("attachment/1q2w3e"), json={"status": True})
        data = client.attachments.delete("1q2w3e")
        assert data is None
        res = client.attachments._last_res
        assert res.url == client._path("attachment/1q2w3e")



from typing import List

import attr

from qaseio.client.models.base import DefaultList


class MimeTypes:
    JS = "application/javascript"
    MJS = "application/javascript"
    JSON = "application/json"
    WEBMANIFEST = "application/manifest+json"
    DOC = "application/msword"
    DOT = "application/msword"
    WIZ = "application/msword"
    BIN = "application/octet-stream"
    A = "application/octet-stream"
    DLL = "application/octet-stream"
    EXE = "application/octet-stream"
    OBJ = "application/octet-stream"
    SO = "application/octet-stream"
    ODA = "application/oda"
    PDF = "application/pdf"
    P7C = "application/pkcs7-mime"
    PS = "application/postscript"
    AI = "application/postscript"
    EPS = "application/postscript"
    M3U = "application/vnd.apple.mpegurl"
    M3U8 = "application/vnd.apple.mpegurl"
    XLS = "application/vnd.ms-excel"
    XLB = "application/vnd.ms-excel"
    PPT = "application/vnd.ms-powerpoint"
    POT = "application/vnd.ms-powerpoint"
    PPA = "application/vnd.ms-powerpoint"
    PPS = "application/vnd.ms-powerpoint"
    PWZ = "application/vnd.ms-powerpoint"
    WASM = "application/wasm"
    BCPIO = "application/x-bcpio"
    CPIO = "application/x-cpio"
    CSH = "application/x-csh"
    DVI = "application/x-dvi"
    GTAR = "application/x-gtar"
    HDF = "application/x-hdf"
    LATEX = "application/x-latex"
    MIF = "application/x-mif"
    CDF = "application/x-netcdf"
    NC = "application/x-netcdf"
    P12 = "application/x-pkcs12"
    PFX = "application/x-pkcs12"
    RAM = "application/x-pn-realaudio"
    PYC = "application/x-python-code"
    PYO = "application/x-python-code"
    SH = "application/x-sh"
    SHAR = "application/x-shar"
    SWF = "application/x-shockwave-flash"
    SV4CPIO = "application/x-sv4cpio"
    SV4CRC = "application/x-sv4crc"
    TAR = "application/x-tar"
    TCL = "application/x-tcl"
    TEX = "application/x-tex"
    TEXI = "application/x-texinfo"
    TEXINFO = "application/x-texinfo"
    ROFF = "application/x-troff"
    T = "application/x-troff"
    TR = "application/x-troff"
    MAN = "application/x-troff-man"
    ME = "application/x-troff-me"
    MS = "application/x-troff-ms"
    USTAR = "application/x-ustar"
    SRC = "application/x-wais-source"
    XSL = "application/xml"
    RDF = "application/xml"
    WSDL = "application/xml"
    XPDL = "application/xml"
    ZIP = "application/zip"
    AU = "audio/basic"
    SND = "audio/basic"
    MP3 = "audio/mpeg"
    MP2 = "audio/mpeg"
    AIF = "audio/x-aiff"
    AIFC = "audio/x-aiff"
    AIFF = "audio/x-aiff"
    RA = "audio/x-pn-realaudio"
    WAV = "audio/x-wav"
    BMP = "image/bmp"
    GIF = "image/gif"
    IEF = "image/ief"
    JPG = "image/jpeg"
    JPE = "image/jpeg"
    JPEG = "image/jpeg"
    PNG = "image/png"
    SVG = "image/svg+xml"
    TIFF = "image/tiff"
    TIF = "image/tiff"
    ICO = "image/vnd.microsoft.icon"
    RAS = "image/x-cmu-raster"
    PNM = "image/x-portable-anymap"
    PBM = "image/x-portable-bitmap"
    PGM = "image/x-portable-graymap"
    PPM = "image/x-portable-pixmap"
    RGB = "image/x-rgb"
    XBM = "image/x-xbitmap"
    XPM = "image/x-xpixmap"
    XWD = "image/x-xwindowdump"
    EML = "message/rfc822"
    MHT = "message/rfc822"
    MHTML = "message/rfc822"
    NWS = "message/rfc822"
    CSS = "text/css"
    CSV = "text/csv"
    HTML = "text/html"
    HTM = "text/html"
    TXT = "text/plain"
    BAT = "text/plain"
    C = "text/plain"
    H = "text/plain"
    KSH = "text/plain"
    PL = "text/plain"
    RTX = "text/richtext"
    TSV = "text/tab-separated-values"
    PY = "text/x-python"
    ETX = "text/x-setext"
    SGM = "text/x-sgml"
    SGML = "text/x-sgml"
    VCF = "text/x-vcard"
    XML = "text/xml"
    MP4 = "video/mp4"
    MPEG = "video/mpeg"
    M1V = "video/mpeg"
    MPA = "video/mpeg"
    MPE = "video/mpeg"
    MPG = "video/mpeg"
    MOV = "video/quicktime"
    QT = "video/quicktime"
    WEBM = "video/webm"
    AVI = "video/x-msvideo"
    MOVIE = "video/x-sgi-movie"


@attr.s
class AttachmentCreated:
    hash = attr.ib(default=None)
    filename = attr.ib(default=None)
    mime = attr.ib(default=None)
    extension = attr.ib(default=None)
    url = attr.ib(default=None)


@attr.s
class AttachmentInfo:
    hash = attr.ib(default=None)
    file = attr.ib(default=None)
    mime = attr.ib(default=None)
    size: int = attr.ib(default=None)
    full_path = attr.ib(default=None)


@attr.s
class AttachmentList(DefaultList):
    entities: List[AttachmentInfo] = attr.ib(factory=list)



import json

import pytest

import requests_mock
from apitist.constructor import converter
from tests.data import _list, _status_true, _test_plan, _test_plan_full

from qaseio.client.models import (
    TestPlanCreate,
    TestPlanCreated,
    TestPlanInfo,
    TestPlanList,
    TestPlanUpdate,
)


@pytest.mark.parametrize(
    "params, query",
    [
        ((10, 30), "?limit=10&offset=30"),
        ((None, 30), "?offset=30"),
        ((10, None), "?limit=10"),
    ],
)
def test_get_all_test_plans(client, params, query):
    response = _status_true(_list(_test_plan()))
    with requests_mock.Mocker() as m:
        m.get(client._path("plan/CODE"), json=response)
        data = client.plans.get_all("CODE", *params)
        assert data == converter.structure(
            response.get("result"), TestPlanList
        )
        res = client.plans._last_res
        assert res.url == client._path("plan/CODE" + query)


def test_get_specific_test_plan(client):
    response = _status_true(_test_plan_full())
    with requests_mock.Mocker() as m:
        m.get(client._path("plan/CODE/123"), json=response)
        data = client.plans.get("CODE", 123)
        assert data == converter.structure(
            response.get("result"), TestPlanInfo
        )
        res = client.plans._last_res
        assert res.url == client._path("plan/CODE/123")


def test_test_plan_exists(client):
    response = _status_true(_test_plan_full())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("plan/CODE/123"),
            [
                {"status_code": 200, "json": response},
                {"status_code": 404, "json": {}},
            ],
        )
        assert client.plans.exists("CODE", 123)
        assert not client.plans.exists("CODE", 123)


def test_create_new_test_plan(client):
    response = _status_true({"id": 123})
    with requests_mock.Mocker() as m:
        m.post(client._path("plan/CODE"), json=response)
        create_data = TestPlanCreate("new test plan", [1, 2, 3])
        data = client.plans.create("CODE", create_data)
        assert data == converter.structure(
            response.get("result"), TestPlanCreated
        )
        res = client.plans._last_res
        assert json.loads(res.request.body) == converter.unstructure(
            create_data
        )


def test_update_test_plan(client):
    response = _status_true({"id": 123})
    with requests_mock.Mocker() as m:
        m.patch(client._path("plan/CODE/123"), json=response)
        update_data = TestPlanUpdate("new test plan", [1, 2, 3])
        data = client.plans.update("CODE", 123, update_data)
        assert data == converter.structure(
            response.get("result"), TestPlanCreated
        )
        res = client.plans._last_res
        assert res.url == client._path("plan/CODE/123")
        assert json.loads(res.request.body) == converter.unstructure(
            update_data
        )


def test_delete_test_plan(client):
    with requests_mock.Mocker() as m:
        m.delete(client._path("plan/CODE/123"), json={"status": True})
        data = client.plans.delete("CODE", 123)
        assert data is None
        res = client.plans._last_res
        assert res.url == client._path("plan/CODE/123")



import mimetypes
import ntpath
from typing import List, Tuple, Union

from qaseio.client.models import (
    AttachmentCreated,
    AttachmentInfo,
    AttachmentList,
)
from qaseio.client.services import BaseService, NotFoundException


class Attachments(BaseService):
    def get_all(self, limit=None, offset=None):
        return self.vr(
            self.s.get(
                self.path("attachment"),
                params={"limit": limit, "offset": offset},
            ),
            to_type=AttachmentList,
        )

    def get(self, hash: str):
        return self.vr(
            self.s.get(self.path("attachment/{}".format(hash))),
            to_type=AttachmentInfo,
        )

    def upload(
        self,
        code: str,
        *file_infos: Union[str, Tuple[str, str], Tuple[bytes, str, str]],
    ):
        files = []
        for _id, file in enumerate(file_infos):
            filename = None
            if isinstance(file, tuple):
                if len(file) == 2:
                    path, mime = file
                else:
                    path, mime, filename = file
            else:
                path = file
                mime = mimetypes.guess_type(file)[0]
            if isinstance(path, bytes):
                content = path
            else:
                content = open(path, "rb")
            files.append(
                (str(_id), (filename or ntpath.basename(path), content, mime))
            )

        return self.vr(
            self.s.post(self.path("attachment/{}".format(code)), files=files),
            to_type=List[AttachmentCreated],
        )

    def delete(self, hash: str):
        return self.vr(
            self.s.delete(self.path("attachment/{}".format(hash))),
            to_type=None,
        )

    def exists(self, code: str):
        try:
            return self.get(code)
        except NotFoundException:
            return False



from enum import Enum, unique
from typing import List

import attr

from qaseio.client.models.base import DefaultFilter, DefaultList


@unique
class TestRunResultStatus(Enum):
    IN_PROGRESS = "in_progress"
    PASSED = "passed"
    FAILED = "failed"
    BLOCKED = "blocked"


@attr.s
class TestRunResultFilters(DefaultFilter):
    status: List[TestRunResultStatus] = attr.ib(default=None)
    member: int = attr.ib(default=None)
    run: int = attr.ib(default=None)
    case_id: int = attr.ib(default=None)
    from_end_time: str = attr.ib(default=None)
    to_end_time: str = attr.ib(default=None)


@attr.s
class TestRunResultInfo:
    hash = attr.ib(default=None)
    comment = attr.ib(default=None)
    stacktrace = attr.ib(default=None)
    run_id = attr.ib(default=None)
    case_id = attr.ib(default=None)
    steps = attr.ib(factory=list)
    status = attr.ib(default=None)
    is_api_result = attr.ib(default=None)
    time_spent = attr.ib(default=None)
    end_time = attr.ib(default=None)
    attachments: List[str] = attr.ib(factory=list)


@attr.s
class TestRunResultList(DefaultList):
    entities: List[TestRunResultInfo] = attr.ib(factory=list)


@attr.s
class TestRunResultStepCreate:
    position: int = attr.ib()
    status: TestRunResultStatus = attr.ib()
    attachments: List[str] = attr.ib(factory=list)
    comment: str = attr.ib(default=None)


@attr.s
class TestRunResultCreate:
    case_id: int = attr.ib()
    status: TestRunResultStatus = attr.ib()
    time: int = attr.ib(default=None)
    time_ms: int = attr.ib(default=None)
    member_id: int = attr.ib(default=None)
    comment: str = attr.ib(default=None)
    stacktrace: str = attr.ib(default=None)
    defect: bool = attr.ib(default=None)
    steps: List[TestRunResultStepCreate] = attr.ib(factory=list)
    attachments: List[str] = attr.ib(factory=list)


@attr.s
class TestRunResultCreated:
    hash: str = attr.ib()


@attr.s
class TestRunResultUpdate:
    status: TestRunResultStatus = attr.ib(default=None)
    time: int = attr.ib(default=None)
    time_ms: int = attr.ib(default=None)
    comment: str = attr.ib(default=None)
    stacktrace: str = attr.ib(default=None)
    defect: bool = attr.ib(default=None)
    steps: List[TestRunResultStepCreate] = attr.ib(factory=list)
    attachments: List[str] = attr.ib(factory=list)



from typing import List

import attr

from qaseio.client.models.base import DefaultFilter, DefaultList


@attr.s
class SharedStepCreate:
    title: str = attr.ib()
    action: str = attr.ib()
    expected_result: str = attr.ib(default=None)


@attr.s
class SharedStepUpdate:
    title: str = attr.ib(default=None)
    action: str = attr.ib(default=None)
    expected_result: str = attr.ib(default=None)


@attr.s
class SharedStepCreated:
    hash: str = attr.ib(default=None)


@attr.s
class SharedStepInfo:
    hash = attr.ib(default=None)
    title = attr.ib(default=None)
    action = attr.ib(default=None)
    expected_result = attr.ib(default=None)
    cases: List[int] = attr.ib(factory=list)
    cases_count: int = attr.ib(default=None)
    created = attr.ib(default=None)
    updated = attr.ib(default=None)


@attr.s
class SharedStepList(DefaultList):
    entities: List[SharedStepInfo] = attr.ib(factory=list)


@attr.s
class SharedStepFilters(DefaultFilter):
    search: str = attr.ib(default=None)



from typing import Union

from qaseio.client.models import (
    DefectFilters,
    DefectInfo,
    DefectList,
    DefectUpdated,
)
from qaseio.client.services import BaseService, NotFoundException


class Defects(BaseService):
    def get_all(
        self,
        code: str,
        limit=None,
        offset=None,
        filters: DefectFilters = None,
    ):
        query = {"limit": limit, "offset": offset}
        if filters:
            query.update(filters.filter())
        return self.vr(
            self.s.get(self.path("defect/{}".format(code)), params=query),
            to_type=DefectList,
        )

    def get(self, code: str, defect_id: Union[str, int]):
        return self.vr(
            self.s.get(self.path("defect/{}/{}".format(code, defect_id))),
            to_type=DefectInfo,
        )

    def resolve(self, code: str, defect_id: Union[str, int]):
        return self.vr(
            self.s.patch(
                self.path("defect/{}/resolve/{}".format(code, defect_id))
            ),
            to_type=DefectUpdated,
        )

    def delete(self, code: str, defect_id: Union[str, int]):
        return self.vr(
            self.s.delete(self.path("defect/{}/{}".format(code, defect_id))),
            to_type=None,
        )

    def exists(self, code: str, defect_id: Union[str, int]):
        try:
            return self.get(code, defect_id)
        except NotFoundException:
            return False



import pytest

import requests_mock
from apitist.constructor import converter
from tests.data import _custom_field, _list, _status_true

from qaseio.client.models import CustomFieldInfo, CustomFieldList


@pytest.mark.parametrize(
    "params, query",
    [
        ((10, 30), "?limit=10&offset=30"),
        ((None, 30), "?offset=30"),
        ((10, None), "?limit=10"),
    ],
)
def test_get_all_custom_fields(client, params, query):
    response = _status_true(_list(_custom_field()))
    with requests_mock.Mocker() as m:
        m.get(client._path("custom_field/CODE"), json=response)
        data = client.custom_fields.get_all("CODE", *params)
        assert data == converter.structure(
            response.get("result"), CustomFieldList
        )
        res = client.custom_fields._last_res
        assert res.url == client._path("custom_field/CODE" + query)


def test_get_specific_custom_field(client):
    response = _status_true(_custom_field())
    with requests_mock.Mocker() as m:
        m.get(client._path("custom_field/CODE/1"), json=response)
        data = client.custom_fields.get("CODE", 1)
        assert data == converter.structure(
            response.get("result"), CustomFieldInfo
        )
        res = client.custom_fields._last_res
        assert res.url == client._path("custom_field/CODE/1")


def test_custom_field_exists(client):
    response = _status_true(_custom_field())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("custom_field/CODE/1"),
            [
                {"status_code": 200, "json": response},
                {"status_code": 404, "json": {}},
            ],
        )
        assert client.custom_fields.exists("CODE", 1)
        assert not client.custom_fields.exists("CODE", 1)



from typing import List

import attr

from qaseio.client.models.base import DefaultList


@attr.s
class UserInfo:
    id = attr.ib(default=None)
    name = attr.ib(default=None)
    email = attr.ib(default=None)
    title = attr.ib(default=None)
    status: int = attr.ib(default=None)


@attr.s
class UserList(DefaultList):
    entities: List[UserInfo] = attr.ib(factory=list)



import pytest

import requests_mock
from apitist.constructor import converter
from tests.data import _defect, _list, _status_true

from qaseio.client.models import (
    DefectFilters,
    DefectInfo,
    DefectList,
    DefectStatus,
    DefectUpdated,
)


@pytest.mark.parametrize(
    "params, query",
    [
        ((10, 30), "?limit=10&offset=30"),
        ((None, 30), "?offset=30"),
        ((10, None), "?limit=10"),
        (
            (
                10,
                None,
                DefectFilters(status=DefectStatus.OPEN),
            ),
            "?limit=10&filters%5Bstatus%5D=open",
        ),
    ],
)
def test_get_all_defects(client, params, query):
    response = _status_true(_list(_defect()))
    with requests_mock.Mocker() as m:
        m.get(client._path("defect/CODE"), json=response)
        data = client.defects.get_all("CODE", *params)
        assert data == converter.structure(response.get("result"), DefectList)
        res = client.defects._last_res
        assert res.url == client._path("defect/CODE" + query)


def test_get_specific_defect(client):
    response = _status_true(_defect())
    with requests_mock.Mocker() as m:
        m.get(client._path("defect/CODE/123"), json=response)
        data = client.defects.get("CODE", 123)
        assert data == converter.structure(response.get("result"), DefectInfo)
        res = client.defects._last_res
        assert res.url == client._path("defect/CODE/123")


def test_defect_exists(client):
    response = _status_true(_defect())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("defect/CODE/123"),
            [
                {"status_code": 200, "json": response},
                {"status_code": 404, "json": {}},
            ],
        )
        assert client.defects.exists("CODE", 123)
        assert not client.defects.exists("CODE", 123)


def test_resolve_defect(client):
    response = _status_true({"id": 123})
    with requests_mock.Mocker() as m:
        m.patch(client._path("defect/CODE/resolve/123"), json=response)
        data = client.defects.resolve("CODE", 123)
        assert data == converter.structure(
            response.get("result"), DefectUpdated
        )


def test_delete_defect(client):
    with requests_mock.Mocker() as m:
        m.delete(client._path("defect/CODE/123"), json={"status": True})
        data = client.defects.delete("CODE", 123)
        assert data is None
        res = client.defects._last_res
        assert res.url == client._path("defect/CODE/123")



import pytest

import requests_mock
from apitist.constructor import converter
from tests.data import _list, _status_true, _test_case

from qaseio.client.models import (
    Severity,
    TestCaseFilters,
    TestCaseInfo,
    TestCaseList,
)


@pytest.mark.parametrize(
    "params, query",
    [
        ((10, 30), "?limit=10&offset=30"),
        ((None, 30), "?offset=30"),
        ((10, None), "?limit=10"),
        (
            (
                10,
                None,
                TestCaseFilters(
                    severity=[Severity.CRITICAL, Severity.BLOCKER]
                ),
            ),
            "?limit=10&filters%5Bseverity%5D=critical%2Cblocker",
        ),
    ],
)
def test_get_all_test_cases(client, params, query):
    response = _status_true(_list(_test_case()))
    with requests_mock.Mocker() as m:
        m.get(client._path("case/CODE"), json=response)
        data = client.cases.get_all("CODE", *params)
        assert data == converter.structure(
            response.get("result"), TestCaseList
        )
        res = client.cases._last_res
        assert res.url == client._path("case/CODE" + query)


def test_get_specific_test_case(client):
    response = _status_true(_test_case())
    with requests_mock.Mocker() as m:
        m.get(client._path("case/CODE/123"), json=response)
        data = client.cases.get("CODE", 123)
        assert data == converter.structure(
            response.get("result"), TestCaseInfo
        )
        res = client.cases._last_res
        assert res.url == client._path("case/CODE/123")


def test_test_case_exists(client):
    response = _status_true(_test_case())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("case/CODE/123"),
            [
                {"status_code": 200, "json": response},
                {"status_code": 404, "json": {}},
            ],
        )
        assert client.cases.exists("CODE", 123)
        assert not client.cases.exists("CODE", 123)


def test_delete_test_case(client):
    with requests_mock.Mocker() as m:
        m.delete(client._path("case/CODE/123"), json={"status": True})
        data = client.cases.delete("CODE", 123)
        assert data is None
        res = client.cases._last_res
        assert res.url == client._path("case/CODE/123")



import json

import pytest

import requests_mock
from apitist.constructor import converter
from tests.data import _list, _status_true, _test_run_result

from qaseio.client.models import (
    TestRunResultCreate,
    TestRunResultCreated,
    TestRunResultFilters,
    TestRunResultInfo,
    TestRunResultList,
    TestRunResultStatus,
    TestRunResultUpdate,
)


@pytest.mark.parametrize(
    "params, query",
    [
        (
            (
                10,
                30,
                TestRunResultFilters(status=[TestRunResultStatus.FAILED]),
            ),
            "?limit=10&offset=30&filters%5Bstatus%5D=failed",
        ),
        ((None, 30, None), "?offset=30"),
        ((10, None, None), "?limit=10"),
        (
            (
                None,
                None,
                TestRunResultFilters(status=[TestRunResultStatus.FAILED]),
            ),
            "?filters%5Bstatus%5D=failed",
        ),
    ],
)
def test_get_all_test_run_results(client, params, query):
    response = _status_true(_list(_test_run_result()))
    with requests_mock.Mocker() as m:
        m.get(client._path("result/CODE"), json=response)
        data = client.results.get_all("CODE", *params)
        assert data == converter.structure(
            response.get("result"), TestRunResultList
        )
        res = client.results._last_res
        assert res.url == client._path("result/CODE" + query)


def test_get_specific_test_run_result(client):
    response = _status_true(_test_run_result())
    with requests_mock.Mocker() as m:
        m.get(client._path("result/CODE/6efce6e4"), json=response)
        data = client.results.get("CODE", "6efce6e4")
        assert data == converter.structure(
            response.get("result"), TestRunResultInfo
        )
        res = client.results._last_res
        assert res.url == client._path("result/CODE/6efce6e4")


def test_create_new_test_run_result(client):
    response = _status_true({"hash": "6efce6e4"})
    with requests_mock.Mocker() as m:
        m.post(client._path("result/CODE/123"), json=response)
        create_data = TestRunResultCreate(123, TestRunResultStatus.BLOCKED)
        data = client.results.create("CODE", 123, create_data)
        assert data == converter.structure(
            response.get("result"), TestRunResultCreated
        )
        res = client.results._last_res
        assert json.loads(res.request.body) == converter.unstructure(
            create_data
        )


def test_update_test_run_result(client):
    response = _status_true({"hash": "6efce6e4"})
    with requests_mock.Mocker() as m:
        m.patch(client._path("result/CODE/123/6efce6e4"), json=response)
        update_data = TestRunResultUpdate(TestRunResultStatus.BLOCKED)
        data = client.results.update("CODE", 123, "6efce6e4", update_data)
        assert data == converter.structure(
            response.get("result"), TestRunResultCreated
        )
        res = client.results._last_res
        assert res.url == client._path("result/CODE/123/6efce6e4")
        assert json.loads(res.request.body) == converter.unstructure(
            update_data
        )


def test_delete_test_run_result(client):
    with requests_mock.Mocker() as m:
        m.delete(
            client._path("result/CODE/123/6efce6e4"), json={"status": True}
        )
        data = client.results.delete("CODE", 123, "6efce6e4")
        assert data is None
        res = client.results._last_res
        assert res.url == client._path("result/CODE/123/6efce6e4")



from enum import Enum, unique
from typing import List

import attr

from qaseio.client.models.base import DefaultFilter, DefaultList


@unique
class DefectStatus(Enum):
    OPEN = "open"
    RESOLVED = "resolved"


@attr.s
class DefectUpdated:
    id = attr.ib(default=None)


@attr.s
class DefectInfo:
    id = attr.ib(default=None)
    title = attr.ib(default=None)
    actual_result = attr.ib(default=None)
    status = attr.ib(default=None)
    user_id: int = attr.ib(default=None)
    attachments: List = attr.ib(factory=list)
    created = attr.ib(default=None)
    updated = attr.ib(default=None)


@attr.s
class DefectList(DefaultList):
    entities: List[DefectInfo] = attr.ib(factory=list)


@attr.s
class DefectFilters(DefaultFilter):
    status: DefectStatus = attr.ib(default=None)



import pytest

import attr
from requests import Response

from qaseio.client.models import ProjectCreated
from qaseio.client.services import BaseService, TooManyRequestsException


@pytest.mark.parametrize("values", [(), (None,)])
def test_empty_service_init(values):
    with pytest.raises(TypeError):
        BaseService(*values)


def test_vr_too_many_requests(base_service):
    res = Response()
    res.status_code = 429
    res._content = b'{"status":true,"result":{}}'
    with pytest.raises(TooManyRequestsException):
        base_service.validate_response(res, to_type=None)


@pytest.mark.parametrize(
    "res_status, validate_status", [(201, 201), (202, [200, 201, 202])]
)
def test_vr_correct_status_code(base_service, res_status, validate_status):
    res = Response()
    res.status_code = res_status
    res._content = b'{"status":true,"result":{}}'
    data = base_service.validate_response(
        res, to_type=None, status=validate_status
    )
    assert data == {}


@pytest.mark.parametrize(
    "res_status, validate_status", [(201, 20), (203, [200, 201, 202])]
)
def test_vr_incorrect_status_code(base_service, res_status, validate_status):
    res = Response()
    res.status_code = res_status
    res._content = b'{"status":true,"result":{}}'
    with pytest.raises(ValueError):
        base_service.validate_response(
            res, to_type=None, status=validate_status
        )


def test_vr_res_status_false(base_service):
    res = Response()
    res.status_code = 200
    res._content = b'{"status":false,"result":{}}'
    with pytest.raises(ValueError):
        base_service.validate_response(res, to_type=None)


def test_vr_res_not_json(base_service):
    res = Response()
    res.status_code = 200
    res._content = b"some none json string"
    with pytest.raises(ValueError):
        base_service.validate_response(res, to_type=None)


def test_vr_res_correct_type(base_service):
    res = Response()
    res.status_code = 200
    res._content = b'{"status":true,"result":{"code":"TRU"}}'
    data = base_service.validate_response(res, to_type=ProjectCreated)
    assert isinstance(data, ProjectCreated)
    assert data.code == "TRU"


def test_vr_res_incorrect_type(base_service):
    @attr.s
    class Ex:
        field = attr.ib()

    res = Response()
    res.status_code = 200
    res._content = b'{"status":true,"result":{"codes":"TRU"}}'
    with pytest.raises(ValueError):
        base_service.validate_response(res, to_type=Ex)



def _project():
    return {
        "title": "Demo Project",
        "code": "DEMO",
        "counts": {
            "cases": 10,
            "suites": 3,
            "milestones": 0,
            "runs": {"total": 1, "active": 1},
            "defects": {"total": 0, "open": 0},
        },
    }


def _test_case():
    return {
        "id": 5,
        "position": 1,
        "title": "Test case",
        "description": "Description for case",
        "preconditions": "",
        "postconditions": "",
        "severity": 4,
        "priority": 2,
        "type": 1,
        "behavior": 2,
        "automation": "is-not-automated",
        "status": "actual",
        "milestone_id": None,
        "suite_id": 1,
        "tags": [],
        "links": [],
        "revision": 1,
        "custom_fields": [],
        "attachments": [],
        "steps": [],
        "created": "2018-05-02T20:32:23.000000Z",
        "updated": "2019-07-21T13:24:08.000000Z",
    }


def _test_run():
    return {
        "id": 1,
        "title": "Test run 2019/12/12",
        "description": None,
        "status": 0,
        "start_time": "2019-12-12 12:12:12",
        "end_time": None,
        "public": False,
        "stats": {
            "total": 3,
            "untested": 3,
            "passed": 0,
            "failed": 0,
            "blocked": 0,
            "skipped": 0,
            "retest": 0,
            "deleted": 4,
        },
        "time_spent": 0,
        "user_id": 1,
        "environment": None,
        "cases": [1, 2, 3],
    }


def _test_plan():
    return {
        "id": 1,
        "title": "Sample plan",
        "description": "Regression",
        "cases_count": 10,
        "created": "2019-01-10T22:47:53.000000Z",
        "updated": "2019-01-10T22:47:53.000000Z",
    }


def _test_plan_full():
    return {
        "id": 16,
        "title": "Sample plan",
        "description": "Awesome plan",
        "cases_count": 1,
        "created": "2019-09-15T18:49:07.000000Z",
        "updated": "2019-09-15T18:49:30.000000Z",
        "average_time": 40,
        "cases": [{"case_id": 1, "assignee_id": 1}],
    }


def _test_suite():
    return {
        "id": 1,
        "title": "Level 1",
        "description": "Set from API",
        "preconditions": "Set from API",
        "position": 1,
        "cases_count": 10,
        "parent_id": None,
        "created": "2018-05-02T10:49:01.000000Z",
        "updated": "2019-07-21T19:10:15.000000Z",
    }


def _shared_step():
    return {
        "hash": "0223905c291bada23e6049d415385982af92d758",
        "title": "Shared step",
        "action": "Open signup page",
        "expected_result": "Page is opened",
        "cases": [41, 35, 42, 30],
        "cases_count": 4,
        "created": "2019-02-09T23:16:49.000000Z",
        "updated": "2019-02-09T23:16:49.000000Z",
    }


def _test_run_result():
    return {
        "hash": "6efce6e4f9de887a2ee863e8197cb74ab626a273",
        "comment": "some comment",
        "stacktrace": "some stacktrace",
        "run_id": 1,
        "case_id": 1,
        "steps": None,
        "status": "Passed",
        "is_api_result": True,
        "time_spent": 0,
        "end_time": "2018-11-11 11:11:11",
        "attachments": [
            "6efce6e4f9de887a2ee863e8197cb74ab626a271",
            "6efce6e4f9de887a2ee863e8197cb74ab626a272",
        ],
    }


def _defect():
    return {
        "id": 1,
        "title": "Dangerous defect",
        "actual_result": "Something happened",
        "status": "open",
        "user_id": 1,
        "attachments": [],
        "created": "2019-11-08T22:03:07.000000Z",
        "updated": "2019-11-19T22:29:57.000000Z",
    }


def _attachment():
    return {
        "hash": "2497be4bc95f807d2fe3c2203793673f6e5140e8",
        "file": "filename.ext",
        "mime": "text/plain",
        "size": 100,
        "full_path": "https://storage.cdn.example/filename.ext",
    }


def _attachment_created():
    return {
        "filename": "qaseio_pytest.py",
        "url": "https://storage.cdn.example/filename.ext",
        "extension": "py",
        "hash": "d81bb2beb147c2eceddbf3e10f98e6216cc757e3",
        "mime": "text\\/x-python",
        "team": "c66dc393c83fe149449e5de3e64545279e1442ed",
    }


def _custom_field():
    return {
        "id": 1,
        "title": "Description",
        "type": "Text",
        "placeholder": "Write something",
        "default_value": None,
        "value": None,
        "is_required": False,
        "is_visible": False,
        "is_filterable": False,
        "created": "2019-08-26T22:30:07.000000Z",
        "updated": "2019-08-26T22:30:07.000000Z",
    }


def _user():
    return {
        "id": 1,
        "name": "John Smith",
        "email": "john@example.com",
        "title": "Team Owner",
        "status": 1,
    }


def _list(data):
    return {"total": 10, "filtered": 10, "count": 1, "entities": [data]}


def _status_true(data):
    return {"status": True, "result": data}



from qaseio.client.models import (
    ProjectCreate,
    ProjectCreated,
    ProjectInfo,
    ProjectList,
)
from qaseio.client.services import BaseService, NotFoundException


class Projects(BaseService):
    def get_all(self, limit=None, offset=None):
        return self.vr(
            self.s.get(
                self.path("project"), params={"limit": limit, "offset": offset}
            ),
            to_type=ProjectList,
        )

    def get(self, code: str):
        return self.vr(
            self.s.get(self.path("project/{}".format(code))),
            to_type=ProjectInfo,
        )

    def create(self, data: ProjectCreate):
        return self.vr(
            self.s.post(self.path("project"), data=data),
            to_type=ProjectCreated,
        )

    def exists(self, code: str):
        try:
            return self.get(code)
        except NotFoundException:
            return False



from typing import Callable, List, Type, TypeVar, Union

import attr
from apitist.constructor import converter
from apitist.requests import Session
from requests import Response


class BadRequestException(Exception):
    pass


class UnauthorizedException(Exception):
    pass


class ForbiddenException(Exception):
    pass


class NotFoundException(Exception):
    pass


class TooManyRequestsException(Exception):
    pass


exceptions = {
    400: BadRequestException,
    401: UnauthorizedException,
    403: ForbiddenException,
    404: NotFoundException,
    429: TooManyRequestsException,
}


T = TypeVar("T")


@attr.s
class BaseService:
    s: Session = attr.ib()
    path: Callable[[str], str] = attr.ib()
    _in_test = attr.ib(default=False)
    _last_res = attr.ib(default=False)

    def validate_response(
        self,
        res: Response,
        to_type: Type[T],
        status: Union[int, List[int]] = 200,
    ) -> T:
        if isinstance(status, int):
            status = [status]

        if self._in_test:
            self._last_res = res

        if res.status_code in exceptions:
            message = "Got error during response: {}".format(res.content)
            raise exceptions[res.status_code](message)

        if res.status_code not in status:
            raise ValueError(
                "Got unexpected status code: {} {}".format(
                    res.status_code, res.content
                )
            )
        try:
            data = res.json()
            if data.get("status") is True and to_type:
                return converter.structure(data.get("result"), to_type)
            elif data.get("status") is True:
                return data.get("result")
        except Exception as e:
            raise ValueError("Unable to parse response {}".format(e))
        raise ValueError("Got error during response: {}".format(res.content))

    vr = validate_response



# flake8: noqa
from .attachments import *
from .base import *
from .cases import *
from .custom_fields import *
from .defects import *
from .milestones import *
from .plans import *
from .projects import *
from .results import *
from .runs import *
from .shared_steps import *
from .suites import *
from .users import *



from enum import Enum, unique
from typing import List

import attr

from qaseio.client.models.base import (
    Automation,
    Behavior,
    DefaultFilter,
    DefaultList,
    Priority,
    Severity,
    Type,
)


@unique
class TestCaseStatus(Enum):
    ACTUAL = "actual"
    DRAFT = "draft"
    DEPRECATED = "deprecated"


@attr.s
class TestCaseInfo:
    id = attr.ib(default=None)
    position = attr.ib(default=None)
    title = attr.ib(default=None)
    description = attr.ib(default=None)
    preconditions = attr.ib(default=None)
    postconditions = attr.ib(default=None)
    severity = attr.ib(default=None)
    priority = attr.ib(default=None)
    type = attr.ib(default=None)
    behavior = attr.ib(default=None)
    automation = attr.ib(default=None)
    status = attr.ib(default=None)
    milestone_id = attr.ib(default=None)
    suite_id = attr.ib(default=None)
    tags = attr.ib(factory=list)
    links = attr.ib(factory=list)
    revision = attr.ib(default=None)
    custom_fields = attr.ib(factory=list)
    attachments = attr.ib(factory=list)
    steps = attr.ib(factory=list)
    created = attr.ib(default=None)
    updated = attr.ib(default=None)


@attr.s
class TestCaseFilters(DefaultFilter):
    search: str = attr.ib(default=None)
    milestone_id: int = attr.ib(default=None)
    suite_id: int = attr.ib(default=None)
    severity: List[Severity] = attr.ib(default=None)
    priority: List[Priority] = attr.ib(default=None)
    type: List[Type] = attr.ib(default=None)
    behavior: List[Behavior] = attr.ib(default=None)
    automation: List[Automation] = attr.ib(default=None)
    status: List[TestCaseStatus] = attr.ib(default=None)


@attr.s
class TestCaseList(DefaultList):
    entities: List[TestCaseInfo] = attr.ib(factory=list)



from typing import List

import attr

from qaseio.client.models.base import DefaultList


@attr.s
class TestPlanCreate:
    title: str = attr.ib()
    cases: List[int] = attr.ib()
    description: str = attr.ib(default=None)

    @cases.validator
    def check(self, _, value):
        if not isinstance(value, list) or len(value) < 1:
            raise ValueError(
                "You should provide at least one test case for a run"
            )


@attr.s
class TestPlanUpdate(TestPlanCreate):
    title: str = attr.ib(default=None)
    cases: List[int] = attr.ib(factory=list)
    description: str = attr.ib(default=None)


@attr.s
class TestPlanCreated:
    id = attr.ib(default=None)


@attr.s
class TestPlanSteps:
    case_id = attr.ib(default=None)
    assignee_id = attr.ib(default=None)


@attr.s
class TestPlanInfo:
    id = attr.ib(default=None)
    title = attr.ib(default=None)
    description = attr.ib(default=None)
    cases_count = attr.ib(default=None)
    created = attr.ib(default=None)
    updated = attr.ib(default=None)
    average_time = attr.ib(default=None)
    cases: List[TestPlanSteps] = attr.ib(factory=list)


@attr.s
class TestPlanList(DefaultList):
    entities: List[TestPlanInfo] = attr.ib(factory=list)



import json

import pytest

import requests_mock
from apitist.constructor import converter
from tests.data import _list, _status_true, _test_run

from qaseio.client.models import (
    TestRunCreate,
    TestRunCreated,
    TestRunFilters,
    TestRunInclude,
    TestRunInfo,
    TestRunList,
    TestRunStatus,
)


@pytest.mark.parametrize(
    "params, query",
    [
        ((10, 30, TestRunInclude.CASES), "?limit=10&offset=30&include=cases"),
        ((None, 30, None), "?offset=30"),
        ((10, None, None), "?limit=10"),
        ((None, None, TestRunInclude.CASES), "?include=cases"),
        (
            (None, None, None, TestRunFilters(status=[TestRunStatus.ABORT])),
            "?filters%5Bstatus%5D=abort",
        ),
    ],
)
def test_get_all_test_runs(client, params, query):
    response = _status_true(_list(_test_run()))
    with requests_mock.Mocker() as m:
        m.get(client._path("run/CODE"), json=response)
        data = client.runs.get_all("CODE", *params)
        assert data == converter.structure(response.get("result"), TestRunList)
        res = client.runs._last_res
        assert res.url == client._path("run/CODE" + query)


def test_get_specific_test_run(client):
    response = _status_true(_test_run())
    with requests_mock.Mocker() as m:
        m.get(client._path("run/CODE/123"), json=response)
        data = client.runs.get("CODE", 123)
        assert data == converter.structure(response.get("result"), TestRunInfo)
        res = client.runs._last_res
        assert res.url == client._path("run/CODE/123")


def test_test_run_exists(client):
    response = _status_true(_test_run())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("run/CODE/123"),
            [
                {"status_code": 200, "json": response},
                {"status_code": 404, "json": {}},
            ],
        )
        assert client.runs.exists("CODE", 123)
        assert not client.runs.exists("CODE", 123)


def test_create_new_test_run(client):
    response = _status_true({"id": 123})
    with requests_mock.Mocker() as m:
        m.post(client._path("run/CODE"), json=response)
        create_data = TestRunCreate("new test run", [1, 2, 3])
        data = client.runs.create("CODE", create_data)
        assert data == converter.structure(
            response.get("result"), TestRunCreated
        )
        res = client.runs._last_res
        assert json.loads(res.request.body) == converter.unstructure(
            create_data
        )


def test_delete_test_run(client):
    with requests_mock.Mocker() as m:
        m.delete(client._path("run/CODE/123"), json={"status": True})
        data = client.runs.delete("CODE", 123)
        assert data is None
        res = client.runs._last_res
        assert res.url == client._path("run/CODE/123")



from typing import Union

from qaseio.client.models import (
    TestRunResultCreate,
    TestRunResultCreated,
    TestRunResultFilters,
    TestRunResultInfo,
    TestRunResultList,
    TestRunResultUpdate,
)
from qaseio.client.services import BaseService


class Results(BaseService):
    def get_all(
        self,
        code: str,
        limit=None,
        offset=None,
        filters: TestRunResultFilters = None,
    ):
        query = {"limit": limit, "offset": offset}
        if filters:
            query.update(filters.filter())
        return self.vr(
            self.s.get(self.path("result/{}".format(code)), params=query),
            to_type=TestRunResultList,
        )

    def get(self, code: str, hash: str):
        return self.vr(
            self.s.get(self.path("result/{}/{}".format(code, hash))),
            to_type=TestRunResultInfo,
        )

    def create(
        self, code: str, run_id: Union[str, int], data: TestRunResultCreate
    ):
        return self.vr(
            self.s.post(
                self.path("result/{}/{}".format(code, run_id)), data=data
            ),
            to_type=TestRunResultCreated,
        )

    def update(
        self,
        code: str,
        run_id: Union[str, int],
        hash: str,
        data: TestRunResultUpdate,
    ):
        return self.vr(
            self.s.patch(
                self.path("result/{}/{}/{}".format(code, run_id, hash)),
                data=data,
            ),
            to_type=TestRunResultCreated,
        )

    def delete(self, code: str, run_id: Union[str, int], hash: str):
        return self.vr(
            self.s.delete(
                self.path("result/{}/{}/{}".format(code, run_id, hash))
            ),
            to_type=None,
        )



import pytest

from qaseio.client.models import (
    AccessLevel,
    ProjectCreate,
    Severity,
    TestCaseFilters,
    TestPlanCreate,
    TestRunCreate,
)


@pytest.mark.parametrize(
    "filters, output",
    [
        (TestCaseFilters(), {}),
        (TestCaseFilters(search="query"), {"filters[search]": "query"}),
        (TestCaseFilters(milestone_id=123), {"filters[milestone_id]": 123}),
        (
            TestCaseFilters(severity=[Severity.BLOCKER, Severity.UNDEFINED]),
            {"filters[severity]": "blocker,undefined"},
        ),
        (
            TestCaseFilters(
                search="query",
                milestone_id=123,
                severity=[Severity.BLOCKER, Severity.UNDEFINED],
            ),
            {
                "filters[search]": "query",
                "filters[milestone_id]": 123,
                "filters[severity]": "blocker,undefined",
            },
        ),
        (
            TestCaseFilters(severity=["blocker", "undefined"]),
            {"filters[severity]": "blocker,undefined"},
        ),
        (
            TestCaseFilters(severity="blocker"),
            {"filters[severity]": "blocker"},
        ),
        (TestCaseFilters(severity=[]), {}),
    ],
)
def test_filters(filters, output):
    assert filters.filter() == output


@pytest.mark.parametrize(
    "data",
    [
        ("", "TE"),
        ("a" * 65, "TESTED", "", AccessLevel.NONE),
        ("test", "TESTED", "a" * 65, AccessLevel.ALL),
        ("test", "TESTED", None, AccessLevel.GROUP, "123414"),
    ],
)
def test_valid_project_creation(data):
    ProjectCreate(*data)


@pytest.mark.parametrize(
    "data",
    [
        ("", "T"),
        ("", None),
        ("a" * 65, "TESTEDS", "", AccessLevel.NONE),
        ("test", "Tes.sd"),
        ("test", "Tes123"),
        ("test", "русский"),
        ("test", "TESTED", None, AccessLevel.GROUP),
    ],
)
def test_invalid_project_creation(data):
    with pytest.raises((ValueError, TypeError)):
        ProjectCreate(*data)


@pytest.mark.parametrize(
    "data",
    [
        ("", [1]),
        ("a" * 65, [13, 14, 15], "a" * 65, 123),
        ("a", []),
        ("a", None),
    ],
)
def test_valid_test_run_creation(data):
    TestRunCreate(*data)


@pytest.mark.parametrize(
    "data",
    [("a", 123), ("a", set())],
)
def test_invalid_test_run_creation(data):
    with pytest.raises(ValueError):
        TestRunCreate(*data)


@pytest.mark.parametrize(
    "data",
    [("", [1]), ("a" * 65, [13, 14, 15], "a" * 65)],
)
def test_valid_test_plan_creation(data):
    TestPlanCreate(*data)


@pytest.mark.parametrize(
    "data",
    [("a", []), ("a", 123)],
)
def test_invalid_test_plan_creation(data):
    with pytest.raises(ValueError):
        TestPlanCreate(*data)



import time

import pytest

import requests_mock
from apitist.constructor import converter
from apitist.hooks import (
    PrepRequestDebugLoggingHook,
    RequestConverterHook,
    ResponseDebugLoggingHook,
)
from tests.data import _project, _status_true

from qaseio.client import QaseApi
from qaseio.client.models import ProjectInfo


def test_session_hooks():
    client = QaseApi("")
    assert RequestConverterHook in client._s.request_hooks
    assert PrepRequestDebugLoggingHook in client._s.prep_request_hooks
    assert ResponseDebugLoggingHook in client._s.response_hooks


def test_retry_request_after(client):
    response = _status_true(_project())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("project/CODE"),
            [
                {"status_code": 429, "headers": {"Retry-After": "5"}},
                {"status_code": 200, "json": response},
            ],
        )
        start_time = time.time()
        data = client.projects.get("CODE")
        stop_time = time.time()
        assert (stop_time - start_time) >= 5
        assert data == converter.structure(response.get("result"), ProjectInfo)


@pytest.mark.parametrize(
    "path, result",
    [
        (None, "https://api.qase.io/v1/"),
        ([], "https://api.qase.io/v1/"),
        ({}, "https://api.qase.io/v1/"),
        ("", "https://api.qase.io/v1/"),
        ("base", "https://api.qase.io/v1/base"),
        ("base/", "https://api.qase.io/v1/base/"),
        ("/base", "https://api.qase.io/v1/base"),
        ("/base/some/123/", "https://api.qase.io/v1/base/some/123/"),
    ],
)
def test_path_func(path, result):
    client = QaseApi("")
    assert client._path(path) == result



import pytest

import requests_mock
from apitist.constructor import converter
from tests.data import _list, _status_true, _user

from qaseio.client.models import UserInfo, UserList


@pytest.mark.parametrize(
    "params, query",
    [
        ((10, 30), "?limit=10&offset=30"),
        ((None, 30), "?offset=30"),
        ((10, None), "?limit=10"),
    ],
)
def test_get_all_users(client, params, query):
    response = _status_true(_list(_user()))
    with requests_mock.Mocker() as m:
        m.get(client._path("user"), json=response)
        data = client.users.get_all(*params)
        assert data == converter.structure(response.get("result"), UserList)
        res = client.users._last_res
        assert res.url == client._path("user" + query)


def test_get_specific_user(client):
    response = _status_true(_user())
    with requests_mock.Mocker() as m:
        m.get(client._path("user/1"), json=response)
        data = client.users.get(1)
        assert data == converter.structure(response.get("result"), UserInfo)
        res = client.users._last_res
        assert res.url == client._path("user/1")


def test_user_exists(client):
    response = _status_true(_user())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("user/1"),
            [
                {"status_code": 200, "json": response},
                {"status_code": 404, "json": {}},
            ],
        )
        assert client.users.exists(1)
        assert not client.users.exists(1)



from typing import Union

from qaseio.client.models import (
    TestSuiteCreate,
    TestSuiteCreated,
    TestSuiteFilters,
    TestSuiteInfo,
    TestSuiteList,
    TestSuiteUpdate,
)
from qaseio.client.services import BaseService, NotFoundException


class Suites(BaseService):
    def get_all(
        self,
        code: str,
        limit=None,
        offset=None,
        filters: TestSuiteFilters = None,
    ):
        query = {"limit": limit, "offset": offset}
        if filters:
            query.update(filters.filter())
        return self.vr(
            self.s.get(self.path("suite/{}".format(code)), params=query),
            to_type=TestSuiteList,
        )

    def get(self, code: str, suite_id: Union[str, int]):
        return self.vr(
            self.s.get(self.path("suite/{}/{}".format(code, suite_id))),
            to_type=TestSuiteInfo,
        )

    def create(self, code: str, data: TestSuiteCreate):
        return self.vr(
            self.s.post(self.path("suite/{}".format(code)), data=data),
            to_type=TestSuiteCreated,
        )

    def update(
        self, code: str, suite_id: Union[str, int], data: TestSuiteUpdate
    ):
        return self.vr(
            self.s.patch(
                self.path("suite/{}/{}".format(code, suite_id)), data=data
            ),
            to_type=TestSuiteCreated,
        )

    def delete(self, code: str, suite_id: Union[str, int]):
        return self.vr(
            self.s.delete(self.path("suite/{}/{}".format(code, suite_id))),
            to_type=None,
        )

    def exists(self, code: str, suite_id: Union[str, int]):
        try:
            return self.get(code, suite_id)
        except NotFoundException:
            return False



import json

import pytest

import requests_mock
from apitist.constructor import converter
from tests.data import _list, _shared_step, _status_true

from qaseio.client.models import (
    SharedStepCreate,
    SharedStepCreated,
    SharedStepFilters,
    SharedStepInfo,
    SharedStepList,
    SharedStepUpdate,
)


@pytest.mark.parametrize(
    "params, query",
    [
        ((10, 30), "?limit=10&offset=30"),
        ((None, 30), "?offset=30"),
        ((10, None), "?limit=10"),
        (
            (
                10,
                None,
                SharedStepFilters(search="123"),
            ),
            "?limit=10&filters%5Bsearch%5D=123",
        ),
    ],
)
def test_get_all_shared_steps(client, params, query):
    response = _status_true(_list(_shared_step()))
    with requests_mock.Mocker() as m:
        m.get(client._path("shared_step/CODE"), json=response)
        data = client.shared_steps.get_all("CODE", *params)
        assert data == converter.structure(
            response.get("result"), SharedStepList
        )
        res = client.shared_steps._last_res
        assert res.url == client._path("shared_step/CODE" + query)


def test_get_specific_shared_step(client):
    response = _status_true(_shared_step())
    with requests_mock.Mocker() as m:
        m.get(client._path("shared_step/CODE/hash"), json=response)
        data = client.shared_steps.get("CODE", "hash")
        assert data == converter.structure(
            response.get("result"), SharedStepInfo
        )
        res = client.shared_steps._last_res
        assert res.url == client._path("shared_step/CODE/hash")


def test_shared_step_exists(client):
    response = _status_true(_shared_step())
    with requests_mock.Mocker() as m:
        m.get(
            client._path("shared_step/CODE/hash"),
            [
                {"status_code": 200, "json": response},
                {"status_code": 404, "json": {}},
            ],
        )
        assert client.shared_steps.exists("CODE", "hash")
        assert not client.shared_steps.exists("CODE", "hash")


def test_create_new_shared_step(client):
    response = _status_true({"hash": "hash"})
    with requests_mock.Mocker() as m:
        m.post(client._path("shared_step/CODE"), json=response)
        create_data = SharedStepCreate("new test", "action")
        data = client.shared_steps.create("CODE", create_data)
        assert data == converter.structure(
            response.get("result"), SharedStepCreated
        )
        res = client.shared_steps._last_res
        assert json.loads(res.request.body) == converter.unstructure(
            create_data
        )


def test_update_shared_step(client):
    response = _status_true({"hash": "hash"})
    with requests_mock.Mocker() as m:
        m.patch(client._path("shared_step/CODE/hash"), json=response)
        update_data = SharedStepUpdate("new test")
        data = client.shared_steps.update("CODE", "hash", update_data)
        assert data == converter.structure(
            response.get("result"), SharedStepCreated
        )
        res = client.shared_steps._last_res
        assert res.url == client._path("shared_step/CODE/hash")
        assert json.loads(res.request.body) == converter.unstructure(
            update_data
        )


def test_delete_shared_step(client):
    with requests_mock.Mocker() as m:
        m.delete(client._path("shared_step/CODE/hash"), json={"status": True})
        data = client.shared_steps.delete("CODE", "hash")
        assert data is None
        res = client.shared_steps._last_res
        assert res.url == client._path("shared_step/CODE/hash")
